__all__ = []

from . import base_exporter
__all__.extend( base_exporter.__all__ )
from .base_exporter import *


from . import constants
__all__.extend( constants.__all__ )
from .constants import *


from . import kringer_df
__all__.extend( kringer_df.__all__ )
from .kringer_df import *


from . import kplot
__all__.extend( kplot.__all__ )
from .kplot import *


from . import ktable
__all__.extend( ktable.__all__ )
from .ktable import *





from Gaugi import load, save,Logger, expandFolders
import numpy as np
import sys,os
import glob

__all__ = [
  'add_tag'
]

def add_tag (path, tag):
  task_list = glob.glob(path+'/*')
  print('There are %i tasks to process.' %(len(task_list)))
  for ifolder in task_list:
    print('Processing: %s' %(ifolder.split('/')[-1]))
    job_file_list = expandFolders( ifolder )
    print('This task contains %i job files.' %(len(job_file_list)))
    for f in job_file_list:
      #print(f)
      raw = load( f )
      raw['tag'] = tag
      for ituned in raw['tunedData']:
        for key1 in ituned['history']['reference'].keys():
          iituned = ituned['history']['reference'][key1]
          #print(iituned.keys()) 
          for key2 in iituned.keys():
            if type(iituned[key2]) is tuple:
              x = list(iituned[key2])
              x[1] = int(x[1])
              iituned[key2] = tuple(x)
      # save the altered file
      save( raw, f)







__all__ = [
    'calc_sp',
    'first_export_tool',
]

import os
import json

import numpy as np
import pandas as pd

from itertools import product 
from Gaugi import load as gload
from Gaugi import save as gsave
from sklearn.metrics import roc_curve
from tensorflow.keras.models import model_from_json

# SP index definition
def calc_sp(pd, fa):
    g_mean = np.sqrt((1-fa) * pd)
    a_mean = ((1-fa) + pd)/2.
    return np.sqrt(g_mean*a_mean)

class first_export_tool(object):
    '''
    This class is the very first export tool to get and prepare the tunings
    to move them for prometheus framework. 

    As in older version of saphyra tunings the thresholds wasn't saved so this class 
    will compute the operation threshold using the data used to train.

    This probably will be changed when saphyra dump the thresholds too.
    '''

    def __init__(self, task_path, operation_dataframe, ringer_data_path, noHAD=False):
        '''
        Arguments:
        - task_path: the path where are all tuned files.
        - operation_dataframe: a .csv file with all models selected to operation.
        - ringer_data: path to ringer_data that can be formated the et and eta.
        - noHAD: noHAD flag to use only EM rings.

        Ex.:
        ringer_data = your_data_path + 'your_file_name...et%i_eta%i.npz'

        '''
        self.task_path         = task_path
        self.op_df            = pd.read_csv(operation_dataframe)
        self.ringer_data_path = ringer_data_path
        self.noHAD            = noHAD
    
    def model_finder(self, tunedfile, model_idx, sort, init):
        '''
        This function will search in the tunedfile for wanted
        model given the model idx, sort and init returning the sequential and the weights.

        Arguments:
        
        - tunedfile: the file with tuned models 
        Ex.: 'tunedDiscr.jobID_0004.pic.gz'

        - model_idx: the model index, this is more convinient when you have many MLP's models.
        - sort: the sort that you want.
        - init: the initialization that you want.
        '''
        for itunedData in tunedfile['tunedData']:
            if ((itunedData['imodel'] == model_idx)\
                and (itunedData['sort'] == sort)\
                and (itunedData['init'] == init)):
                # return the keras sequential and weights
                return itunedData['sequence'], itunedData['weights']
    
    def save_models_dict(self, filename):
        '''
        This function will save the models dictionary obtained in fill_models_dict function.

        Arguments:
        -filename: the name of dictionary to be save.
        '''
        gsave(self.models, '%s.pic.gz' %(filename))

    def fill_models_dict(self):
        '''
        This function will fill the dictionary of operation models using the information
        from the operation dataframe and using the ringer data to calculate the threshold.
        '''
        self.models = {}
        # add the noHAD flag
        self.models['noHAD'] = self.noHAD
        for iet, ieta in product(range(self.op_df.et_bin.nunique()),
                                 range(self.op_df.eta_bin.nunique())):
            print('Processing et bin: %i | eta bin: %i' %(iet, ieta))
            # alocate in the dictionary
            self.models['et%i_eta%i' %(iet, ieta)] = {}

            # 1 step: get the right file, model_id, sort and init to open.
            aux_df = self.op_df.loc[((self.op_df['et_bin'] == iet) &\
                                     (self.op_df['eta_bin'] == ieta)), :].copy()
            f_name, id_model, sort, init = aux_df.iloc[0][['file_name', \
                                                        'model_idx', \
                                                        'sort', \
                                                        'init']].values

            # 2 step: get the model and the weights
            tuned_file = os.path.join(self.task_path, f_name) %(iet, ieta)
            m, w = self.model_finder(tunedfile=gload(tuned_file), model_idx=id_model,
                                sort=sort, init=init)
            # save the model and the weights into the dictionary
            self.models['et%i_eta%i' %(iet, ieta)]['sequential'] = m
            self.models['et%i_eta%i' %(iet, ieta)]['weights']    = w
            # load the keras models in order to get the thresholds
            local_model = model_from_json(json.dumps(m))
            local_model.set_weights(w)

            # 3 step: load the data to predict using the associated model
            print('Load rings data... ')
            data             = gload(self.ringer_data_path %(iet, ieta))
            if self.noHAD: # check if we need all rings or no...
                print('This is a noHAD tuning, using only EM rings...')
                rings            = data['data'][:, 1:89]
            else:
                rings            = data['data'][:, 1:101]
            print('Preprocessing rings...')
            norm             = np.abs(rings.sum(axis=1))
            norm[ norm == 0] = 1 
            rings            = rings/norm[:, None] # normalized rings
            trgt             = data['target']
            # predict on rings
            print('Predict step...')
            nn_output        = local_model.predict(rings)
            print('Getting the operation threshold... ')
            # calculate the thresholds
            fa, pd, thresholds = roc_curve(trgt, nn_output)
            sp = calc_sp(pd, fa)
            # get the knee
            knee = np.argmax(sp)
            # save the threshold
            self.models['et%i_eta%i' %(iet, ieta)]['threshold'] = thresholds[knee]
            print(10*'-')
        print('Done!')
    





__all__ = ['ktable']


from Gaugi.messenger.macros import *
from Gaugi.tex import *
from Gaugi import Logger, expandFolders, load
from functools import reduce
import collections, os, glob, json, copy, re
import numpy as np
import pandas as pd



class ktable( Logger ):

    def __init__(self, config_dict): 
        
        '''
        The objective of this class is extract the tuning information from saphyra's output and
        create a pandas DataFrame using then.
        The informations used in this DataFrame are listed in info_dict, but the user can add more
        information from saphyra summary for example.


        Arguments:
        - path_to_task_files: the path to the files that will be used in the extraction

        Ex.: /volume/v1_task/user.mverissi.*/* 
        the example above will get all the tuned files in the v1_task folder

        - config_dict: a dictionary contains in keys the measures that user want to check and
        the values need to be a empty list.

        Ex.: info = collections.OrderedDict( {
              
              "max_sp_val"      : 'summary/max_sp_val',
              "max_sp_pd_val"   : 'summary/max_sp_pd_val#0',
              "max_sp_fa_val"   : 'summary/max_sp_fa_val#0',
              "max_sp_op"       : 'summary/max_sp_op',
              "max_sp_pd_op"    : 'summary/max_sp_pd_op#0',
              "max_sp_fa_op"    : 'summary/max_sp_fa_op#0',
              'tight_pd_ref'    : "reference/tight_cutbased/pd_ref#0",
              'tight_fa_ref'    : "reference/tight_cutbased/fa_ref#0",
              'tight_pd_ref_passed'     : "reference/tight_cutbased/pd_ref#1",
              'tight_fa_ref_passed'     : "reference/tight_cutbased/fa_ref#1",
              'tight_pd_ref_total'      : "reference/tight_cutbased/pd_ref#2",
              'tight_fa_ref_total'      : "reference/tight_cutbased/fa_ref#2",
              'tight_pd_val_passed'     : "reference/tight_cutbased/pd_val#1",
              'tight_fa_val_passed'     : "reference/tight_cutbased/fa_val#1",
              'tight_pd_val_total'      : "reference/tight_cutbased/pd_val#2",
              'tight_fa_val_total'      : "reference/tight_cutbased/fa_val#2",
              'tight_pd_op_passed'      : "reference/tight_cutbased/pd_op#1",
              'tight_fa_op_passed'      : "reference/tight_cutbased/fa_op#1",
              'tight_pd_op_total'       : "reference/tight_cutbased/pd_op#2",
              'tight_fa_op_total'       : "reference/tight_cutbased/fa_op#2",

              } )

              kt = ktable( path_to_tunings, info, tag = 'v8' )

        - tag: a tag for the tuning

        '''
        Logger.__init__(self)
        self.__config_dict = config_dict
        self.pandas_table = None
        # Check wanted key type 
        if type(config_dict) is not collections.OrderedDict:
          MSG_FATAL( self ,"The wanted key must be an collection.OrderedDict to preserve the order inside of the dataframe.")
        #self.__fill_table( path )



    #
    # Fill the main dataframe with values from the tuning files and convert to pandas dataframe
    #
    def fill(self, basepath, tag):
        '''
        This method will fill the information dictionary and convert then into a pandas DataFrame.
        '''
        paths = expandFolders( basepath )
       
        MSG_INFO(self, "Reading file for %s tag...", tag )

        # Creating the dataframe
        dataframe = collections.OrderedDict({
                              'train_tag'      : [],
                              'et_bin'         : [],
                              'eta_bin'        : [],
                              'model_idx'      : [],
                              'sort'           : [],
                              'init'           : [],
                              'file_name'      : [],
                              'tuned_idx'      : [],
                          })

        # Complete the dataframe for each varname in the config dict
        for varname in self.__config_dict.keys():
            dataframe[varname] = []

        MSG_INFO(self, 'There are %i files for this task...' %(len(paths)))
        MSG_INFO(self, 'Filling the table... ')

        for ituned_file_name in paths:
            gfile = load(ituned_file_name)
            tuned_file = gfile['tunedData']
            
            for idx, ituned in enumerate(tuned_file):
                history = ituned['history']
                # get the basic from model
                dataframe['train_tag'].append(tag)
                dataframe['model_idx'].append(ituned['imodel'])
                dataframe['sort'].append(ituned['sort'])
                dataframe['init'].append(ituned['init'])
                dataframe['et_bin'].append(self.get_etbin(ituned_file_name))
                dataframe['eta_bin'].append(self.get_etabin(ituned_file_name))
                dataframe['file_name'].append(ituned_file_name)
                dataframe['tuned_idx'].append( idx )
                # Get the value for each wanted key passed by the user in the contructor args.
                for key, local  in self.__config_dict.items():
                    dataframe[key].append( self.__get_value( history, local ) )
        
        self.pandas_table = self.pandas_table.append( pd.DataFrame(dataframe) ) if not self.pandas_table is None else pd.DataFrame(dataframe)
        MSG_INFO(self, 'End of fill step, a pandas DataFrame was created...')


    #
    # Get the value using recursive dictionary navigation
    #
    def __get_value(self, history, local):
        '''
        Return the value using recursive navigation
        '''
        # Protection to not override the history since this is a 'mutable' object
        var = copy.copy(history)
        for key in local.split('/'):
            var = var[key.split('#')[0]][int(key.split('#')[1])] if '#' in key else var[key]
        return var


    def get_etbin(self, job):
        '''
        A simple method to get the et bin in for a task.
        Arguments:
        -job: A item of tuned_file_list
        '''
        #print(re.findall(r'et[a]?[0-9]', job))
        #return int(re.findall(r'et[a]?[0-9]', job.split('/')[-2])[0][-1])
        return int(  re.findall(r'et[a]?[0-9]', job)[0][-1] )


    def get_etabin(self, job):
        '''
        A simple method to get the eta bin in for a task.
        Arguments:
        -job: A item of tuned_file_list
        '''
        #print(re.findall(r'et[a]?[0-9]', job))
        #return int(re.findall(r'et[a]?[0-9]', job.split('/')[-2])[1][-1])
        return int( re.findall(r'et[a]?[0-9]',job)[1] [-1] )




    #
    # Get the pandas dataframe
    #
    def get_pandas_table(self):
        '''
        Return the pandas table created in fill_table
        '''
        return self.pandas_table


    #
    # Return only best inits
    #
    def filter_inits(self, key):
        '''
        This method will filter the pandas DataFrame in order to get the best initialization for
        each sort. This filter need a key to ordenate the initialization and filter.
        Arguments:
        -key: a measure to use for filter the initializations

        Ex.: 
        # create a table_info
        Example = table_info(my_task_full_name, my_tuned_info, tag='my_flag')
        # fill table
        Example.fill_table()
        # now filter the inits using a max_sp_val
        my_df = Example.filter_inits(key='max_sp_val')

        In this example my_df is a pandas DataFrame with the best intilization sorted by 'max_sp_val'.
        '''
        return self.pandas_table.loc[self.pandas_table.groupby(['et_bin', 'eta_bin', 'model_idx', 'sort'])[key].idxmax(), :]
   


    #
    # Get the best sorts from best inits table
    #
    def filter_sorts(self, pandas_best_inits, key):
        return pandas_best_inits.loc[pandas_best_inits.groupby(['et_bin', 'eta_bin', 'model_idx'])[key].idxmax(), :]
 


    #
    # Calculate the mean/std table from best inits table
    #
    def describe(self, pandas_best_inits ):
    
        # Create a new dataframe to hold this table
        dataframe = { 'train_tag' : [], 'et_bin' : [], 'eta_bin' : []}
        # Include all wanted keys into the dataframe
        for key in self.__config_dict.keys():
            if 'passed' in key: # Skip counts
                continue
            elif ('op' in key) or ('val' in key):
                dataframe[key+'_mean'] = []; dataframe[key+'_std'] = [];
            else:
                dataframe[key] = []
        
        # Loop over all tuning tags and et/eta bins
        for tag in pandas_best_inits.train_tag.unique():
            for et_bin in pandas_best_inits.et_bin.unique():
                for eta_bin in pandas_best_inits.eta_bin.unique():

                  cv_bin = pandas_best_inits.loc[ (pandas_best_inits.train_tag == tag) & (pandas_best_inits.et_bin == et_bin) & 
                                                  (pandas_best_inits.eta_bin == eta_bin) ]
                  dataframe['train_tag'].append( tag ); dataframe['et_bin'].append( et_bin ); dataframe['eta_bin'].append( eta_bin )
                  for key in self.__config_dict.keys():
                      if 'passed' in key:
                        continue
                      elif ('op' in key) or ('val' in key):
                          dataframe[key+'_mean'].append( cv_bin[key].mean() ); dataframe[key+'_std'].append( cv_bin[key].std() )
                      else:
                          dataframe[key].append( cv_bin[key].unique()[0] )

        # Return the pandas dataframe
        return pd.DataFrame(dataframe)



    #
    # Get tge cross val integrated table from best inits
    #
    def integrate( self, pandas_best_inits, tag ):

        keys = [ key for key in self.__config_dict.keys() if 'passed' in key or 'total' in key]
        table = pandas_best_inits.loc[pandas_best_inits.train_tag==tag].groupby(['sort']).agg(dict(zip( keys, ['sum']*len(keys))))
        for key in keys:
            if 'passed' in key:
                orig_key = key.replace('_passed','')
                values = table[key].div( table[orig_key+'_total'] )
                table[orig_key] = values
                table=table.drop([key],axis=1)
                table=table.drop([orig_key+'_total'],axis=1)  
        
        return table.agg(['mean','std'])



    #
    # Dump all history for each line in the table
    #
    @classmethod
    def dump_all_history( cls, table, output_path , tag):
        if not os.path.exists( output_path ):
          os.mkdir( output_path )
        for _ , row in table.iterrows():
            if row.train_tag != tag:
              continue
            # Load history
            history = load( row.file_name )['tunedData'][row.tuned_idx]['history']
            history['loc'] = {'et_bin' : row.et_bin, 'eta_bin' : row.eta_bin, 'sort' : row.sort, 'model_idx' : row.model_idx}
            name = 'history_et_%i_eta_%i_model_%i_sort_%i.json' % (row.et_bin,row.eta_bin,row.model_idx,row.sort)
            with open(os.path.join(output_path, '%s' %name), 'w') as fp:
                #json.dump(transform_serialize(history), fp)
                json.dump(str(history), fp)



    #
    # Dump the table to csv format
    #
    @classmethod
    def dump_table( cls, table, output_path, table_name):
        '''
        A method to save the pandas DataFrame created by table_info into a .csv file.
        Arguments:
        - cv_table: the table created (and filtered) by table_info;
        - output_path: the file destination;
        - table_name: the file name;

        Ex.:
        # create a table_info
        Example = table_info(my_task_full_name, my_tuned_info, tag='my_flag')
        # fill table
        Example.fill_table()
        # now filter the inits using a max_sp_val
        my_df = Example.filter_inits(key='max_sp_val')
        # now save my_df
        Example.dump_table(my_df, my_path, 'a_very_meaningful_name')

        In this example, a file containing my_df, called 'a_very_meaningful_name.csv', will be saved in my_path
        '''
        table.to_csv(os.path.join(output_path, table_name+'.csv'), index=False)



    #
    # Dump beamer table
    #
    def dump_beamer_table( self, pandas_best_inits, etbins, etabins, operation_points, tags=None ):
       
        cv_table = self.describe( pandas_best_inits )

        # Prepare the config dict using the operation points and some default keys
        config_dict = {}
        for operation_point in operation_points:
            config_dict[operation_point] = {}
            # reference val/op standard keys
            for key in ['pd_val', 'sp_val', 'fa_val', 'pd_op', 'sp_op', 'fa_op']:
                config_dict[operation_point][ key + '_mean' ] = operation_point + '_' + key + '_mean'
                config_dict[operation_point][ key + '_std' ] = operation_point + '_' + key + '_std'
            # reference keys
            config_dict[operation_point][ 'pd_ref' ] = operation_point + '_pd_ref'
            config_dict[operation_point][ 'sp_ref' ] = operation_point + '_sp_ref'
            config_dict[operation_point][ 'fa_ref' ] = operation_point + '_fa_ref'
          
        # Create Latex Et bins
        etbins_str = []; etabins_str = []
        for etBinIdx in range(len(etbins)-1):
            etbin = (etbins[etBinIdx], etbins[etBinIdx+1])
            if etbin[1] > 100 :
                etbins_str.append( r'$E_{T}\text{[GeV]} > %d$' % etbin[0])
            else:
                etbins_str.append(  r'$%d < E_{T} \text{[Gev]}<%d$'%etbin )
      
        # Create Latex eta bins
        for etaBinIdx in range( len(etabins)-1 ):
            etabin = (etabins[etaBinIdx], etabins[etaBinIdx+1])
            etabins_str.append( r'$%.2f<\eta<%.2f$'%etabin )
      
        # Default colors
        colorPD = '\\cellcolor[HTML]{9AFF99}'; colorPF = ''; colorSP = ''

        train_tags = cv_table.train_tag.unique() if not tags else tags
      
        # Dictionary to hold all eff values for each operation point, tag and et/eta binning
        summary = {}
      
        # Create the summary with empty values
        for operation_point in config_dict.keys():
            summary[operation_point] = {}
            for tag in train_tags:
                summary[operation_point][tag] = [ [ {} for __ in range(len(etabins)-1) ] for _ in range(len(etbins)-1)]
                for etBinIdx in range(len(etbins)-1):
                    for etaBinIdx in range(len(etabins)-1):
                        for key in config_dict[operation_point]:
                            summary[operation_point][tag][etBinIdx][etaBinIdx][key] = 0.0
      
        # Fill the summary
        for operation_point in config_dict.keys():
            for tag in train_tags:
                tag_table = cv_table.loc[ cv_table.train_tag == tag]
                for etBinIdx in tag_table.et_bin.unique():
                    for etaBinIdx in tag_table.eta_bin.unique():
                        for key in config_dict[operation_point]:
                            # We have only one line for each tag/et/eta bin
                            summary[operation_point][tag][etBinIdx][etaBinIdx][key] = tag_table.loc[ (tag_table.et_bin==etBinIdx) & 
                                (tag_table.eta_bin==etaBinIdx) ][config_dict[operation_point][key]].values[0]
      

        # Apply beamer
        with BeamerTexReportTemplate1( theme = 'Berlin'
                                 , _toPDF = True
                                 , title = ''
                                 , outputFile = 'test'
                                 , font = 'structurebold' ):
        
            for operation_point in summary.keys():
                ### Prepare tables
                tuning_names = ['']; tuning_names.extend( train_tags )
                lines1 = []
                lines1 += [ HLine(_contextManaged = False) ]
                lines1 += [ HLine(_contextManaged = False) ]
                lines1 += [ TableLine( columns = ['','','kinematic region'] + reduce(lambda x,y: x+y,[['',s,''] for s in etbins_str]), _contextManaged = False ) ]
                lines1 += [ HLine(_contextManaged = False) ]
                lines1 += [ TableLine( columns = ['Det. Region','Method','Type'] + reduce(lambda x,y: x+y,[[colorPD+r'$P_{D}[\%]$',colorSP+r'$SP[\%]$',colorPF+r'$P_{F}[\%]$'] \
                                      for _ in etbins_str]), _contextManaged = False ) ]
                lines1 += [ HLine(_contextManaged = False) ]
      
                for etaBinIdx in range(len(etabins) - 1):
                    for idx, tag in enumerate( train_tags ):
                        cv_values=[]; ref_values=[]
                        for etBinIdx in range(len(etbins) - 1):
                            d = summary[operation_point][tag][etBinIdx][etaBinIdx]
                            sp = d['sp_val_mean']*100; sp_std = d['sp_val_std']*100
                            pd = d['pd_val_mean']*100; pd_std = d['pd_val_std']*100
                            fa = d['fa_val_mean']*100; fa_std = d['fa_val_std']*100
                            refsp = d['sp_ref']; refpd = d['pd_ref']; reffa = d['fa_ref']
                            cv_values   += [ colorPD+('%1.2f$\pm$%1.2f')%(pd,pd_std),colorSP+('%1.2f$\pm$%1.2f')%(sp,sp_std),colorPF+('%1.2f$\pm$%1.2f')%(fa,fa_std),    ]
                            ref_values  += [ colorPD+('%1.2f')%(refpd), colorSP+('%1.2f')%(refsp), colorPF+('%1.2f')%(reffa)]
                        ### Make summary table
                        if idx > 0:
                            lines1 += [ TableLine( columns = ['', tuning_names[idx+1], 'Cross Validation'] + cv_values   , _contextManaged = False ) ]
                        else:
                            lines1 += [ TableLine( columns = ['\multirow{%d}{*}{'%(len(tuning_names))+etabins_str[etaBinIdx]+'}',tuning_names[idx], 'Reference'] + ref_values   , 
                                                _contextManaged = False ) ]
                            lines1 += [ TableLine( columns = ['', tuning_names[idx+1], 'Cross Validation'] + cv_values    , _contextManaged = False ) ]
      
                    lines1 += [ HLine(_contextManaged = False) ]
                lines1 += [ HLine(_contextManaged = False) ]
      

                ### Calculate the final efficiencies
                lines2 = []
                lines2 += [ HLine(_contextManaged = False) ]
                lines2 += [ HLine(_contextManaged = False) ]
                lines2 += [ TableLine( columns = ['',colorPD+r'$P_{D}[\%]$',colorPF+r'$F_{a}[\%]$'], _contextManaged = False ) ]
                lines2 += [ HLine(_contextManaged = False) ]
                for idx, tag in enumerate( train_tags ):
                    itable = self.integrate( pandas_best_inits, tag )
                    pd = itable[operation_point+'_pd_op'].values[0]*100
                    pd_std = itable[operation_point+'_pd_op'].values[1]*100
                    fa = itable[operation_point+'_fa_op'].values[0]*100
                    fa_std = itable[operation_point+'_fa_op'].values[1]*100
                    pdref = itable[operation_point+'_pd_ref'].values[0]*100
                    faref = itable[operation_point+'_fa_ref'].values[0]*100
                    if idx > 0:
                        lines2 += [ TableLine( columns = [tag ,colorPD+('%1.2f$\pm$%1.2f')%(pd,pd_std),colorPF+('%1.2f$\pm$%1.2f')%(fa,fa_std) ], _contextManaged = False ) ]
                    else:
                      lines2 += [ TableLine( columns = ['Ref.' ,colorPD+('%1.2f')%(pdref),colorPF+('%1.2f')%(faref) ], _contextManaged = False ) ]
                      lines2 += [ TableLine( columns = [tag ,colorPD+('%1.2f$\pm$%1.2f')%(pd,pd_std),colorPF+('%1.2f$\pm$%1.2f')%(fa,fa_std) ], _contextManaged = False ) ]


                # Create all tables into the PDF Latex 
                with BeamerSection( name = operation_point.replace('_','\_') ):
                    with BeamerSlide( title = "The Cross Validation Efficiency Values For All Tunings"  ):          
                        with Table( caption = 'The $P_{d}$, $F_{a}$ and $SP $ values for each phase space for each method.') as table:
                            with ResizeBox( size = 1. ) as rb:
                                with Tabular( columns = '|lcc|' + 'ccc|' * len(etbins_str) ) as tabular:
                                    tabular = tabular
                                    for line in lines1:
                                        if isinstance(line, TableLine):
                                            tabular += line
                                        else:
                                            TableLine(line, rounding = None)

                    with BeamerSlide( title = "The General Efficiency"  ):          
                        with Table( caption = 'The general efficiency for the cross validation method for each method.') as table:
                            with ResizeBox( size = 0.7 ) as rb:
                                with Tabular( columns = 'lc' + 'c' * 2 ) as tabular:
                                    tabular = tabular
                                    for line in lines2:
                                        if isinstance(line, TableLine):
                                            tabular += line
                                        else:
                                            TableLine(line, rounding = None)







__all__ = [
    'kringer_df'
]

import os
import json

import numpy as np
import pandas as pd

from tensorflow.keras.models import model_from_json
from Gaugi import Logger
from Gaugi.messenger.macros import *
from Gaugi import load as gload




class kringer_df(Logger):

    def __init__(self, raw_data):
        '''
        This class has the objective of transform the ringer data into pandas Dataframe
        in order to make easer to manipulate and extract information.

        Arguments:
        - raw_data: the file .npz created in prometheus framework
        Ex.:
        my_data_path =data18_path  some_path_to_data/meaninful_data_name_et%i_eta%i.npz')
        et, eta      = 2, 0
        my_raw_data = dict(np.load(my_data_path %(et, eta)))
        '''
        Logger.__init__(self)
        self.raw_data      = raw_data
        self.et_bin        = raw_data['etBinIdx'].tolist()
        self.eta_bin       = raw_data['etaBinIdx'].tolist()
        # create a pandas DataFrame
        MSG_INFO(self, 'Creating a pandas Dataframe... ')
        self.df_           = pd.DataFrame(data=self.raw_data['data'],
                                          columns=self.raw_data['features'])
        self.df_['target'] = self.raw_data['target']

    def get_df(self):
        '''
        This method return the pandas Dataframe.
        '''
        return self.df_
    
    def get_rings(self, noHAD=False):
        '''
        This method will return a pandas Dataframe with the rings.
        The flag noHAD is used to remove the hadronic rings.
        
        Arguments:
        - noHAD: a flag to remove or not the hadronic rings.
        '''
        # decide if want hadronic rings or not
        if noHAD:
            return self.df_[['L2Calo_ring_%i' %(i) for i in range(88)]].copy().values
        else:
            return self.df_[['L2Calo_ring_%i' %(i) for i in range(100)]].copy().values
    
    def add_tuning_decision(self, tuned_models_dict):
        '''
        This method will add into the Dataframe the decision from a given tuning.
        
        The tuned_models_dict can be more the one tuning. The key in the dictionary
        will be the key of the decision in the Dataframe.

        Arguments:
        - tuned_models_dict: the dictionary which contains the tunings that you want add 
        the decision.
        Ex.:
        my_tuned_modes = {
            'model_number1' : some-path-to-the-exported-file-with-this-tuning
        }
        '''

        # loop over the items to build a model dict
        for ikey, ituning_file in tuned_models_dict.items():
            MSG_INFO(self, 'Adding %s model to Dataframe' %(ikey))
            # build the model
            aux_file       = gload(ituning_file)
            local_bin      = aux_file['et%i_eta%i' %(self.et_bin, self.eta_bin)]
            local_model    = model_from_json(json.dumps(local_bin['sequential']))
            local_model.set_weights(local_bin['weights'])        
            threshold      = local_bin['threshold']
            # get rings
            rings          = self.get_rings(noHAD=aux_file['noHAD'])
            rings         /= np.abs(rings.sum(axis=1))[:, None] # normalization step
            local_output   = local_model.predict(rings)
            # create the descision
            local_output[ local_output >= threshold ] = 1. 
            local_output[ local_output < threshold ] = 0.
            # add the decision as a new column into the Dataframe
            self.df_[ikey] = local_output


# Imports
from setuptools import setup, find_packages

# Loading README file
with open("README.md", "r") as fh:
  long_description = fh.read()

setup(
  name = 'kolmov',
  version = '1.0.2',
  license='GPL-3.0',
  description = 'A Framework to perfomance the cross validation for Ringer tunings',
  long_description = long_description,
  long_description_content_type="text/markdown",
  packages=find_packages(),
  author = 'Micael Veríssimo de Araújo, Gabriel Gazola Milan, João Victor da Fonseca Pinto',
  author_email = 'micael.verissimo@lps.ufrj.br, gabriel.milan@lps.ufrj.br, jodafons@lps.ufrj.br',
  url = 'https://github.com/micaelverissimo/kolmov',
  keywords = ['framework', 'validation', 'machine-learning', 'ai', 'plotting', 'data-visualization'],
  install_requires = [
    'keras',
    'numpy>=1.18.1',
    'six>=1.12.0',
    'scipy>=1.4.1',
    'future',
    'sklearn',
    'scikit-learn>=0.22.1',
    'Gaugi>=1.0.0',
    'matplotlib>=3.1.3',
    'seaborn>=0.10.0'
  ],
  classifiers=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
  ],
)



__all__ = [
    'kplot',
    'kquadrant_pocket'
]

import os
import glob
import json
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

from Gaugi import Logger, expandFolders
from Gaugi.messenger.macros import *

from kolmov.core.constants import str_etbins_zee, str_etabins

class kplot( Logger ):

    
    def __init__(self, path, model_idx , str_et_bins = str_etbins_zee, 
                                         str_eta_bins = str_etabins
                                         ):
        
        '''
        This class is a little monitoring tool for saphyra trained models.
        Its use the dumped json history of a trained model. If you don't have this json
        use the dump_all_traiplot_monitoringn_history from base_table in kolmov.
        
        Arguments:
        - path_to_history: the path to json history files

        Ex.: /my_volume/my_history_files

        - model_idx: the index of model that you want extract the monitoring info.
        '''

        Logger.__init__( self )
        self.plot_names = {
                            'loss'          : 'Loss Function Evolution',
                            'max_sp_val'    : r'$SP$ Index Evolution',
                            'max_sp_pd_val' : r'$P_D$ Evolution',
                            'max_sp_fa_val' : r'$F_A$ Evolution',
                            }

        self.h_dict         = self.load(path, model_idx)
        self.sort_name_dict =  {idx : 'Fold %i' %(idx+1) for idx in range(10)}
        self.et_range = str_et_bins
        self.eta_range = str_eta_bins



    def load(self, basepath, model_idx):
        
        '''
        This method will open all the histories that was grouped in the initialize method, 
        and put then into a dictionary in order to make easier to manipulate. Usually used
        for histories dumped from best inists table
        '''
        paths = expandFolders( basepath )
        MSG_INFO( self, "Reading %d files...", len(paths) )
        h_dict = dict()
        for path in paths:
            with open(path) as f:
                obj = dict(eval(json.load(f)))
                key = 'et%d_eta%d_sort_%d' %(obj['loc']['et_bin'], obj['loc']['eta_bin'], obj['loc']['sort'])
                if obj['loc']['model_idx'] != model_idx:
                  continue
                h_dict[key] = obj
        return h_dict




    def plot_training_curves(self, et_bin, eta_bin, plot_path, plot_name):
        '''
        Arguments:
        - et_bin: the energy bin index.
        - eta_bin: the pseudorapidity bin index.
        - plot_path: the path to save the plot.
        - plot_name: the plot name.

        Ex.: 
        # initialize the base_ploter class
        plot_tool = plot_monitoring('my_path/awesome_history', model_idx=0)
        # now plot some bin
        plot_tool.plot_monitoring_curves(et_bin=0,
                                         eta_bin=0,
                                         plot_path=my_save_path, 
                                         plot_name=a_very_meaningful_name)
        '''

        # a easy way to lock only in the necessaries keys
        wanted_h_keys = ['et%i_eta%i_sort_%i' %(et_bin, eta_bin, i) for i in range(10)]
        
        
        # train indicators (today we use only this)
        train_indicators = [('loss', 'val_loss'), 'max_sp_val', 'max_sp_pd_val', 'max_sp_fa_val']
        # create a subplots and fill then.
        fig, ax = plt.subplots( nrows=len(wanted_h_keys),
                                ncols=len(train_indicators),
                                figsize=(25,20))
        fig.subplots_adjust(top=0.95, hspace=0.7)
        # loop over the sorts
        for idx, isort in enumerate(wanted_h_keys):
            isort_k = np.int(isort.split('_')[-1])
            # loop over the train indicators
            for jdx, train_ind in enumerate(train_indicators):
                if isinstance(train_ind, tuple):
                    ax[idx, jdx].set_title('%s - %s' %(self.plot_names[train_ind[0]],
                                                       self.sort_name_dict[isort_k]))
                    ax[idx, jdx].set_xlabel('Epochs')
                    ax[idx, jdx].plot(self.h_dict[isort][train_ind[0]], c='r', label='Train Step')
                    ax[idx, jdx].plot(self.h_dict[isort][train_ind[1]], c='b', label='Validation Step')
                    ax[idx, jdx].legend()
                    ax[idx, jdx].grid()
                else:
                    ax[idx, jdx].set_title('%s - %s' %(self.plot_names[train_ind],
                                                       self.sort_name_dict[isort_k]))
                    ax[idx, jdx].set_xlabel('Epochs')
                    ax[idx, jdx].plot(self.h_dict[isort][train_ind], c='g', label='Validation Step')
                    ax[idx, jdx].grid()
        # set the figure name
        fig.suptitle(r'Monitoring Train Plot - $E_T$: %s | $\eta$: %s'
                     %(self.et_range[et_bin], self.eta_range[eta_bin]), fontsize=20)
        # save figure
        fig.savefig(os.path.join(plot_path, '%s.png' %(plot_name)),
                    dpi=300,
                    bbox_inches='tight')


def kquadrant_pocket(df, plot_configs, output_path='/volume'):
    '''
    This function will provide a simple implementation of prometheus quadrant analysis
    applying conditions on Dataframe.

    Arguments:
    - df: the Dataframe used to extract information;
    - plot_configs: a dictionary which contains the all plot information and the
    condition to produce the plot;
    - outpu_path: the path to save the plot.
    '''
    for ivar in plot_configs.keys():
        print('Plotting %s... ' %(ivar))
        local_config = plot_configs[ivar]
        # loop over variables
        var_cond1 = df[((local_config['cond1']) &\
                        (local_config['common_cond']))][ivar].values
        var_cond1 = var_cond1[((var_cond1 >= local_config['low_edge']) &\
                               (var_cond1 <= local_config['high_edge']))]

        var_cond2 = df[((local_config['cond2']) &\
                        (local_config['common_cond']))][ivar].values 
        var_cond2 = var_cond2[((var_cond2 >= local_config['low_edge']) &\
                               (var_cond2 <= local_config['high_edge']))]

        fig, (ax1, ax2) = plt.subplots(nrows=2, figsize=(8, 6))

        fig.suptitle(r'%s Distribution Comparision - %s | %s' %(local_config['var_name'],
                                                                local_config['title_tag1'],
                                                                local_config['title_tag2']),
                    fontsize=20)

        ns, bins, patches =  ax1.hist([var_cond1, var_cond2], normed=False,
                                    bins=local_config['nbins'],
                                    color=['blue', 'red'],
                                    histtype='step', 
                                    alpha=.5,
                                    label=[local_config['cond1_label'],
                                           local_config['cond2_label']],
                                    lw=2)
        ax1.legend(loc='upper left')

        # ratio plot - vulgo puchadinho
        ax2.plot(bins[:-1],     # this is what makes it comparable
                100*(ns[1] / ns[0]), # maybe check for div-by-zero!
                '^', 
                color='black')


        ax1.set_ylabel('Counts', fontsize=15)
        ax2.set_ylabel('Ratio', fontsize=15)
        ax1.set_yscale('log')
        ax2.set_yscale('log')
        ax2.set_xlabel(r'%s' %(local_config['var_name']), fontsize=15)
        # at this time we don't need this
        # ax1.set_xticks(np.arange(start=-0.05, stop=0.06, step=1e-2))
        # ax2.set_xticks(np.arange(start=-0.05, stop=0.06, step=1e-2))
        ax1.set_xlim([local_config['low_edge'], local_config['high_edge']])
        ax2.set_xlim([local_config['low_edge'], local_config['high_edge']])
        ax2.grid()
        ax1.grid()
        plt.savefig(os.path.join(output_path,
                                '%s_mini_quad.png' %(ivar)),
                    dpi=300)